/** 
* Copyright (c) 2022-2024. jinggezhihui. All rights reserved.  
* FileName: DeviceInfo.java 
* Author: marunming
* Email: runming56@163.com
* Date: 2023年10月26日 下午3:05:40
* version: V1.0
* LastModified: 2023年10月26日 下午3:05:40   
*/ 


package com.jg.softbus.device;

import com.jg.softbus.network.protocol.ConnectionAddr;

import java.io.Serializable;
import java.util.Objects;

/** 
* 设备信息
*
* @Author marunming
* @ClassName: DeviceInfo 
* @Email runming56@163.com
* @Date 2023年10月26日 下午3:05:40 
*/
public class DeviceInfo implements Serializable {

	/**
	 * 
	 */
	private static final long serialVersionUID = 2026921411515550451L;
	/** Device ID. Its maximum length is 96. */
	private String devId;
	/** Account hash code. Its maximum length is 96. */
	private String accountHash;
	/** Device type. For details, see {@link DeviceType}. */
	private DeviceType devType;
	/** Device name. Its maximum length is 65. */
	private String devName;
	/** Device Online Status **/
	private boolean isOnline;
	/** Number of available connections */
	private int addrNum;
	/** Connection information. For details, see {@link ConnectionAddr}. */
	private ConnectionAddr[] addr;
	/** Number of capabilities */
	private int capabilityBitmapNum;
	/** Device capability bitmap.
	 * The maximum number of capabilities in the bitmap is 2.
	 */
	private String[] capabilityBitmap;
	/** Custom data. Its length is DISC_MAX_CUST_DATA_LEN. */
	private String custData;
	/** The distance of discovered device, in centimeters(cm)*/
	private int range;

	public String getDevId() {
		return devId;
	}

	public void setDevId(String devId) {
		this.devId = devId;
	}

	public String getAccountHash() {
		return accountHash;
	}

	public void setAccountHash(String accountHash) {
		this.accountHash = accountHash;
	}

	public DeviceType getDevType() {
		return devType;
	}

	public void setDevType(DeviceType devType) {
		this.devType = devType;
	}

	public String getDevName() {
		return devName;
	}

	public void setDevName(String devName) {
		this.devName = devName;
	}

	public boolean getOnline() {
		return isOnline;
	}

	public void setOnline(boolean online) {
		isOnline = online;
	}

	public int getAddrNum() {
		return addrNum;
	}

	public void setAddrNum(int addrNum) {
		this.addrNum = addrNum;
	}

	public ConnectionAddr[] getAddr() {
		return addr;
	}

	public void setAddr(ConnectionAddr[] addr) {
		this.addr = addr;
	}

	public int getCapabilityBitmapNum() {
		return capabilityBitmapNum;
	}

	public void setCapabilityBitmapNum(int capabilityBitmapNum) {
		this.capabilityBitmapNum = capabilityBitmapNum;
	}

	public String[] getCapabilityBitmap() {
		return capabilityBitmap;
	}

	public void setCapabilityBitmap(String[] capabilityBitmap) {
		this.capabilityBitmap = capabilityBitmap;
	}

	public String getCustData() {
		return custData;
	}

	public void setCustData(String custData) {
		this.custData = custData;
	}

	public int getRange() {
		return range;
	}

	public void setRange(int range) {
		this.range = range;
	}

	@Override
	public boolean equals(Object o) {
		if (this == o) return true;
		if (o == null || getClass() != o.getClass()) return false;
		DeviceInfo that = (DeviceInfo) o;
		return Objects.equals(devId, that.devId);
	}

}
