package com.jg.softbus.discovery.enums;

/**
 * Enumerates supported capabilities published by a device.
 */
public enum DataBitMap {
    /**
     * HiCall能力位图,
     * HiCall是一种通信存储解决方案，它可以用于电话录音、语音存储和呼叫中心等场景
     */
    HICALL_CAPABILITY_BITMAP(0, "hicall"),
    /**
     * Profile能力位图,
     * 它可以用于存储和管理用户的个人配置文件，包括个人设置、偏好和应用程序数据等
     */
    PROFILE_CAPABILITY_BITMAP(1, "profile"),
    /**
     * HomeVisionPic能力位图
     * HomeVisionPic是一种家庭视觉存储解决方案，用于存储和管理家庭照片、视频和其他多媒体内容
     */
    HOMEVISIONPIC_CAPABILITY_BITMAP(2, "homevisionPic"),
    /**
     * CastPlus能力位图
     * CastPlus是一种广播存储解决方案，用于存储和管理广播内容，包括音频、视频和广播节目等
     */
    CASTPLUS_CAPABILITY_BITMAP(3, "castPlus"),
    /**
     * AA能力位图
     * 它可以用于自动化地存储和管理数据归档，包括备份、归档和长期存储等
     */
    AA_CAPABILITY_BITMAP(4, "aaCapability"),
    /**
     * DVKIT能力位图
     * DVKIT是一种开发工具包，用于构建和管理存储解决方案，包括数据管理、存储管理和访问控制等
     */
    DVKIT_CAPABILITY_BITMAP(5, "dvKit"),
    /**
     * DDMP能力位图
     * DDMP是一种分布式数据管理平台，用于存储和管理大规模分布式数据，包括数据存储、数据访问和数据处理等
     */
    DDMP_CAPABILITY_BITMAP(6, "ddmpCapability"),
    /**
     * OSD能力位图
     *
     * 它表示OSD支持的各种功能和特性，如数据加密、数据压缩、数据去重、快照、复制等
     */
    OSD_CAPABILITY_BITMAP(7, "osdCapability"),
    /**
     * Share能力位图
     * 它可以用于多用户或多应用程序之间共享存储资源，实现数据共享和协作
     */
    SHARE_CAPABILITY_BITMAP(8, "share"),
    /**
     * Approach能力位图
     * 它表示存储系统或软件所支持的功能和特性，
     * 如存储资源管理、数据迁移、性能优化、容错机制等。
     * 它是用于确定整个存储系统的能力，并根据需要进行相应的配置和操作
     */
    APPROACH_CAPABILITY_BITMAP(9, "approach");

    private final int value;

    private final String abbr;

    DataBitMap(int value, String abbr) {
        this.value = value;
        this.abbr = abbr;
    }

    public int valueOf() {
        return value;
    }

    public String getAbbr() {
        return abbr;
    }

    public static DataBitMap getForValue(int value) {
        //根据值返回枚举
        for (DataBitMap dataBitMap : DataBitMap.values()) {
            if (dataBitMap.valueOf() == value) {
                return dataBitMap;
            }
        }
        return null;
    }

}
