package com.jg.softbus.discovery.enums;

import java.util.Arrays;
import java.util.Optional;

/**
 * Enumerates media, such as Bluetooth, Wi-Fi, and USB, used for publishing services.
 * Currently, only <b>COAP</b> is supported.
 * When <b>AUTO</b> is selected, all the supported media will be called automatically.
 */
public enum ExchangeMedium {
    /** Automatic medium selection */
    AUTO(0),
    /** Bluetooth */
    BLE(1),
    /** Wi-Fi */
    COAP(2),
    /** USB */
    USB(3),
    /** HiLink */
    COAP1(4),
    MEDIUM_BUTT(5);

    private final int value;

    ExchangeMedium(int value) {
        this.value = value;
    }

    public int valueOf() {
        return value;
    }

    public static ExchangeMedium getForValue(int value) {
        Optional<ExchangeMedium> rs =
                Arrays.stream(ExchangeMedium.values()).filter(e -> e.value == value).findAny();
        return rs.orElse(null);
    }
}
