package com.jg.softbus.network.callback;

import com.jg.softbus.network.node.NodeBasicInfo;
import com.jg.softbus.network.node.NodeBasicInfoType;
import com.jg.softbus.network.node.NodeStatus;
import com.jg.softbus.network.node.NodeStatusType;

/**
 * 网络节点变化监听接口
 */
public interface INodeStateCb {
    // 事件掩码
    static final int EVENT_NODE_STATE_ONLINE = 0x1;
    static final int EVENT_NODE_STATE_OFFLINE = 0x02;
    static final int EVENT_NODE_STATE_INFO_CHANGED = 0x04;
    /**
     * 3.2以上版本启用
     * @since 3.2
     */
    static final int EVENT_NODE_STATUS_CHANGED = 0x08;
    /**
     * EVENT_NODE_STATE_MASK 3.2以后为0xF
     */
    static final int EVENT_NODE_STATE_MASK = 0x07;

    default int getEvents() {
        return EVENT_NODE_STATE_MASK;
    }

    /**
     * 节点上线事件回调
     * @param info
     */
    void onNodeOnline(NodeBasicInfo info);

    /**
     * 节点下线事件回调
     * @param info
     */
    void onNodeOffline(NodeBasicInfo info);

    /**
     * 节点信息变化事件回调
     * @param type
     * @param info
     */
    void onNodeBasicInfoChanged(NodeBasicInfoType type, NodeBasicInfo info);

    /**
     * 设备运行状态变化事件回调
     * @param type
     * @since 3.2
     */
    void onNodeStatusChanged(NodeStatusType type, NodeStatus status);
}
