package com.jg.softbus.service;

import com.jg.softbus.discovery.PublishInfo;
import com.jg.softbus.discovery.SubscribeInfo;
import com.jg.softbus.discovery.callback.IDiscoveryCallback;
import com.jg.softbus.discovery.callback.IPublishCallback;
import com.jg.softbus.manager.SessionManager;
import com.jg.softbus.model.SessionInfo;
import com.jg.softbus.session.SessionAttribute;
import com.jg.softbus.session.StreamData;
import com.jg.softbus.session.StreamFrameInfo;
import com.jg.softbus.session.callback.IFileReceiveListener;
import com.jg.softbus.session.callback.IFileSendListener;

import java.io.IOException;
import java.util.List;

public interface ISoftbusService {
    /**
     * 启动服务
     * 基于旧有API启动服务
     * @return
     */
    boolean start() throws Exception;

    /**
     * 启动服务
     * 基于新的API启动服务
     * @return
     * @throws Exception
     */
    boolean startNew() throws Exception;

    /**
     * 获取session管理器
     * @return
     */
    SessionManager getSessionManager();
    /**
     * 发送消息到所有在线终端
     * @param message
     * @return
     * @throws Exception
     */
    boolean sendMsg(String message) throws Exception;

    /**
     * 发送数据到指定session
     * @param data
     * @param session
     * @return
     * @throws Exception
     */
    boolean sendMsg(byte[] data, SessionInfo session) throws Exception;
    /**
     * 发送消息到指定终端
     * @param message
     * @param sessionId
     * @return
     * @throws Exception
     */
    boolean sendMsg(String message, int sessionId) throws Exception;

    /**
     * 发送数据到指定session
     * @param data
     * @param session
     * @param attr
     * @return
     * @throws Exception
     */
    boolean sendMsg(byte[] data, SessionInfo session, SessionAttribute attr) throws Exception;

    /**
     * 发送流到指定session
     * @param sessionId
     * @param data
     * @param ext
     * @param param
     * @return
     */
    boolean sendStream(int sessionId, StreamData data, StreamData ext, StreamFrameInfo param) throws Exception;

    /**
     * 设置文件发送监听器
     * @param fileSendListener
     * @throws Exception
     */
    void setFileSendListener(IFileSendListener fileSendListener) throws Exception;
    /**
     * 发送文件到指定session
     * @param sessionId
     * @param sFileList
     * @param dFileList
     * @param fileCnt
     * @return
     */
    boolean sendFile(int sessionId, String[] sFileList, String[] dFileList, int fileCnt) throws Exception;

    /**
     * 设置文件接收监听器
     * @param fileReceiveListener
     * @param directory
     * @return
     * @throws Exception
     */
    void setFileReceiveListener(IFileReceiveListener fileReceiveListener, String directory) throws Exception;
    /**
     * 停止服务
     * @return
     */
    boolean stopService() throws IOException;
}
